﻿function Get-WmiEventConsumer
{
	<#
	.Synopsis
	Retrieves WMI event consumer objects.
	
	.Description
	Retrieves WMI event consumers instances based on criteria passed to the function. The -Namespace All parameter value can be used to retrieve instances in all WMI namespaces on a given computer.
	
	.Link
	http://trevorsullivan.net
	
	.Link
	http://powershell.artofshell.com
	#>
	
	[CmdletBinding(
		  SupportsShouldProcess = $false
		, SupportsTransactions  = $false
		, ConfirmImpact         = 'Low'
	)]

	param (
		[parameter(
			  Mandatory   = $true
			, HelpMessage = "Please specify the name of event consumer you would like to retrieve."
		)]
		[string]
		${Name}
		,
		[string]
		${Namespace} = 'root\subscription'
		,
		# In the interest of think + type, I've adjusted these types from their actual WMI class names
		#   EventLog    = NTEventLogEventConsumer
		#   LogFile     = LogFileEventConsumer 
		#   Script      = ActiveScriptEventConsumer
		#   CommandLine = CommandLineEventConsumer
		#   SMTP        = SMTPEventConsumer
		[parameter(
			  Mandatory   = $true
			, HelpMessage = "Please specify the type of event consumer you would like to retrieve."
		)]
		[ValidateSet(
			  "EventLog"
			, "LogFile"
			, "CommandLine"
			, "Script"
			, "SMTP"
		)]
		[alias("Type")]
		${ConsumerType}
	)
	
	Begin
	{
		# Get the cmdlet name for writing dynamic log messages
		${CmdletName} = $Pscmdlet.MyInvocation.MyCommand.Name

		$ConsumerClasses = @{
			Script        = "ActiveScriptEventConsumer"
			; SMTP        = "SMTPEventConsumer"
			; EventLog    = "NTEventLogEventConsumer"
			; LogFile     = "LogFileEventConsumer"
			; CommandLine = "CommandLineEventConsumer"
		}
	}

	Process
	{
		if (${Namespace} -ne 'All')
		{
			# Translate asterisks (wildcards) to percent signs (WMI wildcards)
			${Name} = ${Name}.Replace("*", "%")

			# $ConsumerList is an array that holds a list of WMI event consumers returned from WMI.
			# If multiple namespaces are queries for consumers, this will consolidate the results into a single variable.
			$ConsumerList = @()

			${ConsumerQuery} = "select * from $($ConsumerClasses.$ConsumerType) where Name like '${Name}'"
			Write-Verbose ("${CmdletName}: Event filter query is: " + ${ConsumerQuery})
			${EventFilters} = Get-WmiObject -Query ${ConsumerQuery} -Namespace ${Namespace}
			
			if (${EventFilters})
			{
				Write-Verbose ("${CmdletName}: Retrieved " + $Filters.Count + " event consumers from the ${Namespace} namespace.")
			}
			else
			{
				Write-Verbose ("${CmdletName}: Could not find any consumers with the specified name and type.")
			}
				
			
			Write-Output ${EventFilters}
		}
		else
		{
			
		}
	}
}

Export-ModuleMember -Function Get-WmiEventConsumer